<?php

/**
 * Ekomi
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magentocommerce.com so we can send you a copy immediately.
 *
 * @category    Ekomi
 * @package     Ekomi_EkomiIntegration
 * @copyright   Copyright (c) 2018 Ekomi ltd (http://www.ekomi.de)
 * @license     http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *
 * Class Ekomi_EkomiIntegration_Model_Observer
 */
class Ekomi_EkomiIntegration_Model_Observer
{
    /**
     * Constants used in class
     */
    const PD_ORDERS_API_URL = 'https://plugins-dashboard.ekomiapps.de/api/v1/order';
    const STORE_CONFIG_PATH = 'trans_email/ident_sales';
    const EVENT_CRON   = '1';
    const EVENT_UPDATE = '2';

    /**
     * Executes every day by Cron job and exports order data to eKomi
     *
     * @return void
     */
    public function exportOrdersCron()
    {
        $helper = Mage::helper('ekomi_ekomiIntegration');
        $stores = Mage::app()->getStores();
        foreach ($stores as $store) {
            $storeId = Mage::app()->getStore($store)->getId();
            if ($helper->isModuleEnabled($storeId) && $helper->getExportMethod($storeId) == self::EVENT_CRON && $helper->isTermsAndConditionsAccepted($storeId)) {
                $statuses = explode(',', $helper->getOrderStatusForReviewEmail($storeId));
                if (is_array($statuses) && !empty($statuses)) {
                    $orders = Mage::getModel('sales/order')
                        ->getCollection()
                        ->addFieldToFilter(
                            'created_at',
                            array(
                                'from' => strtotime(
                                    "-{$helper->getTurnaroundTime()} day",
                                    Mage::getModel('core/date')->timestamp(time($storeId))
                                ),
                                'to' => Mage::getModel('core/date')->timestamp(time()),
                                'datetime' => true
                            )
                        )->addFieldToFilter(
                            'store_id', $storeId
                        )->addFieldToFilter(
                            'status', $statuses
                        );

                    $this->exportOrders($storeId, $orders);
                }
            }
        }
    }

    /**
     * Iterates through all the orders of a shop and Sends them to eKomi
     * 
     * @param int $storeId
     * @param array $orders
     * @return bool
     */
    private function exportOrders($storeId, $orders = array())
    {
        if (!empty($orders)) {
            foreach ($orders as $order) {
                try {
                    $orderDataJson = $this->getData($order, $storeId);
                    if (!empty($orderDataJson)) {
                        $this->sendOrderData($orderDataJson);
                    }
                } catch (Exception $e) {
                    Mage::logException($e);
                }
            }
        }

        return true;
    }

    /**
     * Sends order to eKomi System (Triggers on Order Status Change Event)
     *
     * @param Mage_Sales_Model_Observer $observer
     * @return void
     */
    public function sendOrderToEkomi($observer)
    {
        $order = $observer->getEvent()->getOrder();
        $storeId = $order->getStoreId();
        $helper = Mage::helper('ekomi_ekomiIntegration');
        $statuses = explode(',', $helper->getOrderStatusForReviewEmail($storeId));

        if ($helper->isModuleEnabled($storeId) && $helper->getExportMethod($storeId) == self::EVENT_UPDATE && $helper->isTermsAndConditionsAccepted($storeId)) {
            if (is_array($statuses) && !empty($statuses) && in_array($order->getStatus(), $statuses)) {
                try {
                    $orderDataJson = $this->getData($order, $storeId);
                    if ($orderDataJson != '') {
                        $this->sendOrderData($orderDataJson);
                    }
                } catch (Exception $e) {
                    Mage::logException($e);
                }
            }
        }
    }

    /**
     * Collects plugin configuration and order data
     *
     * @param $order
     * @param $storeId
     * @return string
     */
    protected function getData($order, $storeId)
    {
        $helper = Mage::helper('ekomi_ekomiIntegration');

        $data = array(
            'shop_id' => $helper->getShopId($storeId),
            'interface_password' => $helper->getShopPassword($storeId),
            'order_data' => $this->getOrderData($order),
            'mode' => $helper->getReviewMod($storeId),
            'product_identifier'=> $helper->getProductIdentifier($storeId),
            'exclude_products'   => $helper->getExcludeProducts($storeId),
            'product_reviews' => (int) $helper->isProductReviewEnabled($storeId),
            'plugin_name' => 'magento'
        );

        return json_encode($data);
    }

    /**
     * Extracts Required fields from order object
     *
     * @param $order
     * @return array
     */
    protected function getOrderData($order)
    {
        $addressData  = $this->getAddressData($order);
        $productsData = $this->getProductsData($order);

        return array (
            'transaction_id' => $order->getIncrementId(),
            'transaction_time' => Mage::getModel('core/date')->date(
                'd-m-Y H:i:s',
                $order->getCreatedAtStoreDate()
            ),
            'last_updated' => Mage::getModel('core/date')->date(
                'd-m-Y H:i:s',
                $order->getUpdatedAtStoreDate()
            ),
            'status' => $order->getStatus(),
            'store_id' => $order->getStoreId(),
            'email' => $order->getCustomerEmail(),
            'customer_id' => $order->getCustomerId() ? $order->getCustomerId() : 'guest_' . $order->getIncrementId(),
            'address' => $addressData,
            'products' => $productsData,
            'sender_name'  => Mage::getStoreConfig(self::STORE_CONFIG_PATH . '/name'),
            'sender_email' => Mage::getStoreConfig(self::STORE_CONFIG_PATH . '/email'),
        );
    }

    /**
     * Extracts Required address fields from order object
     *
     * @param $order
     * @return array
     */
    protected function getAddressData($order)
    {
        $address = $order->getShippingAddress();
        if (!$address) {
            $address = $order->getBillingAddress();
        }

        return array (
            'first_name' => $address->getFirstname(),
            'last_name' => $address->getLastname(),
            "telephone" => $address->getTelephone(),
            "region" => $address->getRegion(),
            "postcode" => $address->getPostcode(),
            "street" => $address->getStreet(),
            "city" => $address->getCity(),
            "country" => $address->getCountry(),
            "address_type" => $address->getAddressType(),
        );
    }

    /**
     * Extracts Required product fields from order object
     *
     * @param $order
     * @return array
     */
    protected function getProductsData($order)
    {
        $products = array();
        $items = $order->getAllVisibleItems();
        foreach ($items as $item) {
            $product = $item->getProduct();

            $imageUrl = '';
            $canonicalUrl = $product->getUrlModel()->getUrl($product, array('_ignore_category' => true));
            try {
                $imageUrl = Mage::getSingleton('catalog/product_media_config')->getMediaUrl($product->getThumbnail());
            } catch (Exception $e) {
                Mage::logException($e);
            }

            $products[] = array(
                'id' => $product->getId(),
                'sku' => $product->getSku(),
                'type' => $product->getTypeId(),
                'name' => $product->getName(),
                'description' => $product->getDescription(),
                'price' => $product->getPrice(),
                'url' => $product->getUrl(),
                'image_url' => $imageUrl,
                'canonicalUrl' => $canonicalUrl,
            );
        }

        return $products;
    }


    /**
     * Sends order data to Plugin Dashboard
     *
     * @param $orderDataJson
     * @return bool|string
     */
    protected function sendOrderData($orderDataJson)
    {
        $boundary = sha1(Mage::getModel('core/date')->timestamp(time()));

        return Mage::helper('ekomi_ekomiIntegration')->apiRequest(
            Zend_Http_Client::PUT,
            self::PD_ORDERS_API_URL,
            array('ContentType:multipart/form-data;boundary=' . $boundary),
            $orderDataJson
        );
    }

}
